import asyncio
import base64
from contextlib import asynccontextmanager
import json
import typing
import httpx
import websockets
import websockets.protocol
from json.decoder import JSONDecodeError

from hume.core.websocket import (
    OnErrorHandlerType,
    OnMessageHandlerType,
    OnOpenCloseHandlerType,
)
from hume.empathic_voice.chat.types.publish_event import PublishEvent
from hume.empathic_voice.types.pause_assistant_message import PauseAssistantMessage
from hume.empathic_voice.types.resume_assistant_message import ResumeAssistantMessage
from hume.empathic_voice.types.tool_error_message import ToolErrorMessage
from hume.empathic_voice.types.tool_response_message import ToolResponseMessage

from ..chat.types.subscribe_event import SubscribeEvent
from ..types.assistant_input import AssistantInput
from ..types.session_settings import SessionSettings
from ..types.audio_input import AudioInput
from ..types.user_input import UserInput
from ...core.pydantic_utilities import parse_obj_as
from ...core.client_wrapper import AsyncClientWrapper
from ...core.api_error import ApiError


class ChatConnectOptions(typing.TypedDict, total=False):
    config_id: typing.Optional[str]
    """
    The ID of the configuration.
    """

    config_version: typing.Optional[str]
    """
    The version of the configuration.
    """

    api_key: typing.Optional[str]

    secret_key: typing.Optional[str]


class ChatWebsocketConnection:
    DEFAULT_NUM_CHANNELS: typing.ClassVar[int] = 1
    DEFAULT_SAMPLE_RATE: typing.ClassVar[int] = 44_100

    def __init__(
        self,
        *,
        websocket: websockets.WebSocketClientProtocol,
    ):
        self.websocket = websocket

        self._num_channels = self.DEFAULT_NUM_CHANNELS
        self._sample_rate = self.DEFAULT_SAMPLE_RATE

    async def __aiter__(self):
        async for message in self.websocket:
            yield parse_obj_as(SubscribeEvent, json.loads(message))  # type: ignore

    async def _send(self, data: typing.Any) -> None:
        if isinstance(data, dict):
            data = json.dumps(data)
        await self.websocket.send(data)

    async def recv(self) -> SubscribeEvent:
        data = await self.websocket.recv()
        return parse_obj_as(SubscribeEvent, json.loads(data))  # type: ignore

    async def _send_model(self, data: PublishEvent) -> None:
        await self._send(data.dict())

    async def send_audio_input(self, message: AudioInput) -> None:
        """
        Parameters
        ----------
        message : AudioInput

        Returns
        -------
        None
        """
        await self._send_model(message)

    async def send_session_settings(self, message: SessionSettings) -> None:
        """
        Update the EVI session settings.

        Parameters
        ----------
        message : SessionSettings

        Returns
        -------
        None
        """

        # Update sample rate and channels
        if message.audio is not None:
            if message.audio.channels is not None:
                self._num_channels = message.audio.channels
            if message.audio.sample_rate is not None:
                self._sample_rate = message.audio.sample_rate

        await self._send_model(message)

    async def send_user_input(self, message: UserInput) -> None:
        """
        Parameters
        ----------
        message : UserInput

        Returns
        -------
        None
        """
        await self._send_model(message)

    async def send_assistant_input(self, message: AssistantInput) -> None:
        """
        Parameters
        ----------
        message : AssistantInput

        Returns
        -------
        None
        """
        await self._send_model(message)

    async def send_tool_response(self, message: ToolResponseMessage) -> None:
        """
        Parameters
        ----------
        message : ToolResponseMessage

        Returns
        -------
        None
        """
        await self._send_model(message)

    async def send_tool_error(self, message: ToolErrorMessage) -> None:
        """
        Parameters
        ----------
        message : ToolErrorMessage

        Returns
        -------
        None
        """
        await self._send_model(message)

    async def send_pause_assistant(self, message: PauseAssistantMessage) -> None:
        """
        Parameters
        ----------
        message : PauseAssistantMessage

        Returns
        -------
        None
        """
        await self._send_model(message)

    async def send_resume_assistant(self, message: ResumeAssistantMessage) -> None:
        """
        Parameters
        ----------
        message : ResumeAssistantMessage

        Returns
        -------
        None
        """
        await self._send_model(message)


class AsyncChatClientWithWebsocket:
    DEFAULT_MAX_PAYLOAD_SIZE_BYTES: typing.ClassVar[int] = 2**24

    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self.client_wrapper = client_wrapper

    async def _construct_ws_uri(self, options: typing.Optional[ChatConnectOptions]):
        query_params = httpx.QueryParams()

        api_key = self.client_wrapper.api_key
        if options is not None:
            maybe_api_key = options.get("api_key")
            if maybe_api_key is not None:
                api_key = maybe_api_key
            maybe_config_id = options.get("config_id")
            if maybe_config_id is not None:
                query_params = query_params.add("config_id", maybe_config_id)
            maybe_config_version = options.get("config_version")
            if maybe_config_version is not None:
                query_params = query_params.add(
                    "config_version", maybe_config_version
                )
            maybe_secret_key = options.get("secret_key")
            if maybe_secret_key is not None and api_key is not None:
                query_params = query_params.add(
                    "accessToken",
                    await self._fetch_access_token(maybe_secret_key, api_key),
                )
            elif api_key is not None:
                query_params = query_params.add("apiKey", api_key)
        elif api_key is not None:
            query_params = query_params.add("apiKey", api_key)

        return f"wss://api.hume.ai/v0/evi/chat?{query_params}"

    @asynccontextmanager
    async def connect(
        self, options: typing.Optional[ChatConnectOptions] = None
    ) -> typing.AsyncIterator[ChatWebsocketConnection]:
        ws_uri = await self._construct_ws_uri(options)

        try:
            async with websockets.connect(
                ws_uri,
                extra_headers=self.client_wrapper.get_headers(include_auth=False),
                max_size=self.DEFAULT_MAX_PAYLOAD_SIZE_BYTES,
            ) as protocol:
                yield ChatWebsocketConnection(websocket=protocol)
        except websockets.exceptions.InvalidStatusCode as exc:
            status_code: int = exc.status_code
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    body="Websocket initialized with invalid credentials.",
                ) from exc
            raise ApiError(
                status_code=status_code,
                body="Unexpected error when initializing websocket connection.",
            ) from exc

    async def _wrap_on_open_close(
        self, on_open: typing.Optional[OnOpenCloseHandlerType]
    ):
        if on_open is not None:
            if asyncio.iscoroutinefunction(on_open):
                await on_open()
            else:
                on_open()

    async def _wrap_on_error(
        self, exc: Exception, on_error: typing.Optional[OnErrorHandlerType]
    ) -> None:
        if on_error is not None:
            if asyncio.iscoroutinefunction(on_error):
                await on_error(exc)
            else:
                on_error(exc)

    async def _wrap_on_message(
        self,
        message: SubscribeEvent,
        on_message: typing.Optional[OnMessageHandlerType[SubscribeEvent]],
    ) -> None:
        if on_message is not None:
            if asyncio.iscoroutinefunction(on_message):
                await on_message(message)
            else:
                on_message(message)

    async def _process_connection(
        self,
        connection: ChatWebsocketConnection,
        on_message: typing.Optional[OnMessageHandlerType],
        on_error: typing.Optional[OnErrorHandlerType],
    ) -> None:
        async for message in connection:
            try:
                await self._wrap_on_message(message, on_message)
            except Exception as exc:
                await self._wrap_on_error(exc, on_error)

    @asynccontextmanager
    async def connect_with_callbacks(
        self,
        options: typing.Optional[ChatConnectOptions] = None,
        on_open: typing.Optional[OnOpenCloseHandlerType] = None,
        on_message: typing.Optional[OnMessageHandlerType[SubscribeEvent]] = None,
        on_close: typing.Optional[OnOpenCloseHandlerType] = None,
        on_error: typing.Optional[OnErrorHandlerType] = None,
    ) -> typing.AsyncIterator[ChatWebsocketConnection]:
        """
        Parameters
        ----------
        on_open : Optional[OnOpenCloseHandlerType]
            A callable to be invoked on the opening of the websocket connection.

        on_message : Optional[OnMessageHandlerType[SubscribeEvent]]
            A callable to be invoked on receiving a message from the websocket connection. This callback should expect a `SubscribeEvent` object.

        on_close : Optional[OnOpenCloseHandlerType]
            A callable to be invoked on the closing of the websocket connection.

        on_error : Optional[OnErrorHandlerType]
            A callable to be invoked on receiving an error from the websocket connection.

        Yields
        -------
        AsyncIterator[ChatWebsocketConnection]
        """

        ws_uri = await self._construct_ws_uri(options)
        background_task: typing.Optional[asyncio.Task[None]] = None

        try:
            async with websockets.connect(
                ws_uri,
                extra_headers=self.client_wrapper.get_headers(include_auth=False),
                max_size=self.DEFAULT_MAX_PAYLOAD_SIZE_BYTES,
            ) as protocol:
                await self._wrap_on_open_close(on_open)
                connection = ChatWebsocketConnection(websocket=protocol)
                background_task = asyncio.create_task(
                    self._process_connection(connection, on_message, on_error)
                )

                yield connection

        # Special case authentication errors
        except websockets.exceptions.InvalidStatusCode as exc:
            status_code: int = exc.status_code
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    body="Websocket initialized with invalid credentials.",
                ) from exc
            raise ApiError(
                status_code=status_code,
                body="Unexpected error when initializing websocket connection.",
            ) from exc

        # Except all other errors to apply the on_error handler
        except Exception as exc:
            await self._wrap_on_error(exc, on_error)
            raise

        # Finally, apply the on_close handler
        finally:
            if background_task is not None:
                background_task.cancel()
                try:
                    await background_task
                except asyncio.CancelledError:
                    pass
            await self._wrap_on_open_close(on_close)

    async def _fetch_access_token(self, secret_key: str, api_key: str) -> str:
        auth = f"{api_key}:{secret_key}"
        encoded_auth = base64.b64encode(auth.encode()).decode()
        _response = await self.client_wrapper.httpx_client.request(
            method="POST",
            base_url="https://api.hume.ai/",
            path="oauth2-cc/token",
            headers={"Authorization": f"Basic {encoded_auth}"},
            data={"grant_type": "client_credentials"},
        )

        if 200 <= _response.status_code < 300:
            return _response.json()["access_token"]
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



"""Abstract base class for model configurations."""

from abc import ABC, abstractmethod
from dataclasses import dataclass

from hume.legacy._common.config_base import ConfigBase
from hume.legacy.models import ModelType


@dataclass
class ModelConfigBase(ConfigBase["ModelConfigBase"], ABC):
    """Abstract base class for model configurations."""

    @classmethod
    @abstractmethod
    def get_model_type(cls) -> ModelType:
        """Get the configuration model type.

        Returns:
            ModelType: Model type.
        """



# This file was auto-generated by Fern from our API Definition.

from ....core.pydantic_utilities import UniversalBaseModel
import pydantic
from .inference_request import InferenceRequest
from .state_inference import StateInference
from ....core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class JobInference(UniversalBaseModel):
    job_id: str = pydantic.Field()
    """
    The ID associated with this job.
    """

    request: InferenceRequest = pydantic.Field()
    """
    The request that initiated the job.
    """

    state: StateInference = pydantic.Field()
    """
    The current state of the job.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .return_active_chat_count_per_tag import ReturnActiveChatCountPerTag
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ReturnActiveChatCount(UniversalBaseModel):
    """
    A description of current chat chat sessions for a user
    """

    timestamp: int = pydantic.Field()
    """
    The timestamp for when chat status was measured. Formatted as a Unix epoch milliseconds.
    """

    total_user_active_chats: int = pydantic.Field()
    """
    The total number of active chats for this user.
    """

    max_allowed_active_chats: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum number of concurrent active chats for this user.
    """

    more_active_chats_allowed: bool = pydantic.Field()
    """
    Boolean indicating if the user is allowed to start more chats.
    """

    per_tag: typing.Optional[typing.List[typing.Optional[ReturnActiveChatCountPerTag]]] = pydantic.Field(default=None)
    """
    Optional List of chat counts per tag.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from .posted_custom_voice_base_voice import PostedCustomVoiceBaseVoice
import typing
from .posted_custom_voice_parameters import PostedCustomVoiceParameters
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class PostedCustomVoice(UniversalBaseModel):
    """
    A Custom Voice specification to be associated with this Config.

    If a Custom Voice specification is not provided then the [name](/reference/empathic-voice-interface-evi/configs/create-config#request.body.voice.name) of a base voice or previously created Custom Voice must be provided.

    See our [Voices guide](/docs/empathic-voice-interface-evi/voices) for a tutorial on how to craft a Custom Voice.
    """

    name: str = pydantic.Field()
    """
    The name of the Custom Voice. Maximum length of 75 characters. Will be converted to all-uppercase. (e.g., "sample voice" becomes "SAMPLE VOICE")
    """

    base_voice: PostedCustomVoiceBaseVoice = pydantic.Field()
    """
    Specifies the base voice used to create the Custom Voice.
    """

    parameter_model: typing.Literal["20241004-11parameter"] = pydantic.Field(default="20241004-11parameter")
    """
    The name of the parameter model used to define which attributes are used by the `parameters` field. Currently, only `20241004-11parameter` is supported as the parameter model.
    """

    parameters: typing.Optional[PostedCustomVoiceParameters] = pydantic.Field(default=None)
    """
    The specified attributes of a Custom Voice.
    
    If no parameters are specified then all attributes will be set to their defaults, meaning no modfications will be made to the base voice.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



"""Async client for handling messages to and from an EVI connection."""

import asyncio
import logging
from dataclasses import dataclass

from hume.empathic_voice.chat.audio.asyncio_utilities import Stream
from hume.empathic_voice.chat.audio.audio_utilities import play_audio
from hume.empathic_voice.chat.audio.microphone_sender import Sender
from hume.empathic_voice.chat.socket_client import ChatWebsocketConnection

logger = logging.getLogger(__name__)

@dataclass
class ChatClient:
    """Async client for handling messages to and from an EVI connection."""

    sender: Sender
    byte_strs: Stream[bytes]

    @classmethod
    def new(cls, *, sender: Sender, byte_strs: Stream[bytes]) -> "ChatClient":
        """Create a new chat client.

        Args:
            sender (_Sender): Sender for audio data.
            byte_strs (Stream[bytes]): Byte stream of audio data. 
        """
        return cls(sender=sender, byte_strs=byte_strs)

    async def _play(self) -> None:
        async for byte_str in self.byte_strs:
            await self.sender.on_audio_begin()
            await play_audio(byte_str)
            await self.sender.on_audio_end()

    async def run(self, *, socket: ChatWebsocketConnection) -> None:
        """Run the chat client.

        Args:
            socket (ChatWebsocketConnection): EVI socket.
        """
        send = self.sender.send(socket=socket)

        await asyncio.gather(self._play(), send)


# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from .return_custom_voice_base_voice import ReturnCustomVoiceBaseVoice
import typing
from .return_custom_voice_parameters import ReturnCustomVoiceParameters
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ReturnCustomVoice(UniversalBaseModel):
    """
    A Custom Voice specification associated with this Config.
    """

    id: str = pydantic.Field()
    """
    Identifier for a Custom Voice. Formatted as a UUID.
    """

    version: int = pydantic.Field()
    """
    Version number for a Custom Voice.
    
    Custom Voices, Prompts, Configs, and Tools are versioned. This versioning system supports iterative development, allowing you to progressively refine configurations and revert to previous versions if needed.
    
    Version numbers are integer values representing different iterations of the Custom Voice. Each update to the Custom Voice increments its version number.
    """

    name: str = pydantic.Field()
    """
    The name of the Custom Voice. Maximum length of 75 characters.
    """

    created_on: int = pydantic.Field()
    """
    Time at which the Custom Voice was created. Measured in seconds since the Unix epoch.
    """

    modified_on: int = pydantic.Field()
    """
    Time at which the Custom Voice was last modified. Measured in seconds since the Unix epoch.
    """

    base_voice: ReturnCustomVoiceBaseVoice = pydantic.Field()
    """
    The base voice used to create the Custom Voice.
    """

    parameter_model: typing.Literal["20241004-11parameter"] = pydantic.Field(default="20241004-11parameter")
    """
    The name of the parameter model used to define which attributes are used by the `parameters` field. Currently, only `20241004-11parameter` is supported as the parameter model.
    """

    parameters: ReturnCustomVoiceParameters = pydantic.Field()
    """
    The specified attributes of a Custom Voice. If a parameter's value is `0` (default), it will not be included in the response.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import aiofiles
import typing
import json as jsonlib
from json.decoder import JSONDecodeError

from ...core.request_options import RequestOptions
from ...core.jsonable_encoder import jsonable_encoder
from ... import core

from .types.inference_base_request import InferenceBaseRequest
from ...core.pydantic_utilities import parse_obj_as
from .types.job_id import JobId
from .client import AsyncBatchClient, BatchClient
from ...core.api_error import ApiError

class BatchClientWithUtils(BatchClient):
    def get_and_write_job_artifacts(
            self,
            id: str,
            *,
            file_name: str = "artifacts.zip",
            request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Get the artifacts ZIP of a completed inference job.

        Parameters
        ----------
        id : str
            The unique identifier for the job.

        file_name : typing.Optional[str]
            The name of the file to write the artifacts to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[bytes]


        Examples
        --------
        from hume.client import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.expression_measurement.batch.get_and_write_job_artifacts(
            id="string",
            file_name="artifacts.zip",
        )
        """
        with open(file_name, mode='wb') as f:
            for chunk in self.get_job_artifacts(id=id, request_options=request_options):
                f.write(chunk)

    def start_inference_job_from_local_file(
        self,
        *,
        file: typing.List[core.File],
        json: typing.Optional[InferenceBaseRequest] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> str:
        """
        Start a new batch inference job.

        Parameters
        ----------
        file : typing.List[core.File]
            See core.File for more documentation

        json : typing.Optional[InferenceBaseRequest]
            The inference job configuration.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str


        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.expression_measurement.batch.start_inference_job_from_local_file()
        """
        files: typing.Dict[str, typing.Any] = {
            "file": file,
        }
        if json is not None:
            files["json"] = jsonlib.dumps(jsonable_encoder(json)).encode("utf-8")

        _response = self._client_wrapper.httpx_client.request(
            "v0/batch/jobs",
            method="POST",
            files=files,
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    JobId,
                    parse_obj_as(
                        type_=JobId,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return _parsed_response.job_id
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncBatchClientWithUtils(AsyncBatchClient):
    async def get_and_write_job_artifacts(
            self,
            id: str,
            *,
            file_name: str = "artifacts.zip",
            request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Get the artifacts ZIP of a completed inference job.

        Parameters
        ----------
        id : str
            The unique identifier for the job.

        file_name : typing.Optional[str]
            The name of the file to write the artifacts to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[bytes]


        Examples
        --------
        from hume.client import HumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )
        await client.expression_measurement.batch.get_and_write_job_artifacts(
            id="string",
            file_name="artifacts.zip",
        )
        """
        async with aiofiles.open(file_name, mode='wb') as f:
            async for chunk in self.get_job_artifacts(id=id, request_options=request_options):
                await f.write(chunk)

    async def start_inference_job_from_local_file(
        self,
        *,
        file: typing.List[core.File],
        json: typing.Optional[InferenceBaseRequest] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> str:
        """
        Start a new batch inference job.

        Parameters
        ----------
        file : typing.List[core.File]
            See core.File for more documentation

        json : typing.Optional[InferenceBaseRequest]
            The inference job configuration.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str


        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.expression_measurement.batch.start_inference_job_from_local_file()
        """
        files: typing.Dict[str, typing.Any] = {
            "file": file,
        }
        if json is not None:
            files["json"] = jsonlib.dumps(jsonable_encoder(json)).encode("utf-8")

        _response = await self._client_wrapper.httpx_client.request(
            "v0/batch/jobs",
            method="POST",
            files=files,
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    JobId,
                    parse_obj_as(
                        type_=JobId,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return _parsed_response.job_id
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from .assistant_end import AssistantEnd
from .assistant_message import AssistantMessage
from .chat_metadata import ChatMetadata
from .web_socket_error import WebSocketError
from .user_interruption import UserInterruption
from .user_message import UserMessage
from .tool_call_message import ToolCallMessage
from .tool_response_message import ToolResponseMessage
from .tool_error_message import ToolErrorMessage

JsonMessage = typing.Union[
    AssistantEnd,
    AssistantMessage,
    ChatMetadata,
    WebSocketError,
    UserInterruption,
    UserMessage,
    ToolCallMessage,
    ToolResponseMessage,
    ToolErrorMessage,
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    Config,
    EmotionEmbedding,
    EmotionEmbeddingItem,
    JobDetails,
    Sentiment,
    SentimentItem,
    StreamBoundingBox,
    StreamErrorMessage,
    StreamFace,
    StreamLanguage,
    StreamModelPredictions,
    StreamModelPredictionsBurst,
    StreamModelPredictionsBurstPredictionsItem,
    StreamModelPredictionsFace,
    StreamModelPredictionsFacePredictionsItem,
    StreamModelPredictionsFacemesh,
    StreamModelPredictionsFacemeshPredictionsItem,
    StreamModelPredictionsJobDetails,
    StreamModelPredictionsLanguage,
    StreamModelPredictionsLanguagePredictionsItem,
    StreamModelPredictionsProsody,
    StreamModelPredictionsProsodyPredictionsItem,
    StreamModelsEndpointPayload,
    StreamWarningMessage,
    StreamWarningMessageJobDetails,
    SubscribeEvent,
    TextPosition,
    TimeRange,
    Toxicity,
    ToxicityItem,
)

__all__ = [
    "Config",
    "EmotionEmbedding",
    "EmotionEmbeddingItem",
    "JobDetails",
    "Sentiment",
    "SentimentItem",
    "StreamBoundingBox",
    "StreamErrorMessage",
    "StreamFace",
    "StreamLanguage",
    "StreamModelPredictions",
    "StreamModelPredictionsBurst",
    "StreamModelPredictionsBurstPredictionsItem",
    "StreamModelPredictionsFace",
    "StreamModelPredictionsFacePredictionsItem",
    "StreamModelPredictionsFacemesh",
    "StreamModelPredictionsFacemeshPredictionsItem",
    "StreamModelPredictionsJobDetails",
    "StreamModelPredictionsLanguage",
    "StreamModelPredictionsLanguagePredictionsItem",
    "StreamModelPredictionsProsody",
    "StreamModelPredictionsProsodyPredictionsItem",
    "StreamModelsEndpointPayload",
    "StreamWarningMessage",
    "StreamWarningMessageJobDetails",
    "SubscribeEvent",
    "TextPosition",
    "TimeRange",
    "Toxicity",
    "ToxicityItem",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .tool_type import ToolType
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ToolCallMessage(UniversalBaseModel):
    """
    When provided, the output is a tool call.
    """

    name: str = pydantic.Field()
    """
    Name of the tool called.
    """

    parameters: str = pydantic.Field()
    """
    Parameters of the tool.
    
    These parameters define the inputs needed for the tool’s execution, including the expected data type and description for each input field. Structured as a stringified JSON schema, this format ensures the tool receives data in the expected format.
    """

    tool_call_id: str = pydantic.Field()
    """
    The unique identifier for a specific tool call instance.
    
    This ID is used to track the request and response of a particular tool invocation, ensuring that the correct response is linked to the appropriate request.
    """

    type: typing.Literal["tool_call"] = pydantic.Field(default="tool_call")
    """
    The type of message sent through the socket; for a Tool Call message, this must be `tool_call`.
    
    This message indicates that the supplemental LLM has detected a need to invoke the specified tool.
    """

    custom_session_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Used to manage conversational state, correlate frontend and backend data, and persist conversations across EVI sessions.
    """

    tool_type: typing.Optional[ToolType] = pydantic.Field(default=None)
    """
    Type of tool called. Either `builtin` for natively implemented tools, like web search, or `function` for user-defined tools.
    """

    response_required: bool = pydantic.Field()
    """
    Indicates whether a response to the tool call is required from the developer, either in the form of a [Tool Response message](/reference/empathic-voice-interface-evi/chat/chat#send.Tool%20Response%20Message.type) or a [Tool Error message](/reference/empathic-voice-interface-evi/chat/chat#send.Tool%20Error%20Message.type).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class UserInterruption(UniversalBaseModel):
    """
    When provided, the output is an interruption.
    """

    type: typing.Literal["user_interruption"] = pydantic.Field(default="user_interruption")
    """
    The type of message sent through the socket; for a User Interruption message, this must be `user_interruption`.
    
    This message indicates the user has interrupted the assistant’s response. EVI detects the interruption in real-time and sends this message to signal the interruption event. This message allows the system to stop the current audio playback, clear the audio queue, and prepare to handle new user input.
    """

    custom_session_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Used to manage conversational state, correlate frontend and backend data, and persist conversations across EVI sessions.
    """

    time: int = pydantic.Field()
    """
    Unix timestamp of the detected user interruption.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from .state_embedding_generation_queued import StateEmbeddingGenerationQueued
from .state_embedding_generation_in_progress import StateEmbeddingGenerationInProgress
from .state_embedding_generation_completed_embedding_generation import (
    StateEmbeddingGenerationCompletedEmbeddingGeneration,
)
from .state_embedding_generation_failed import StateEmbeddingGenerationFailed

StateEmbeddingGeneration = typing.Union[
    StateEmbeddingGenerationQueued,
    StateEmbeddingGenerationInProgress,
    StateEmbeddingGenerationCompletedEmbeddingGeneration,
    StateEmbeddingGenerationFailed,
]



"""Interface for connecting a device microphone to an EVI connection."""

from __future__ import annotations

import logging
from dataclasses import dataclass
from typing import ClassVar

from hume.legacy._voice.microphone.chat_client import ChatClient
from hume.legacy._voice.microphone.microphone import Microphone
from hume.legacy._voice.microphone.microphone_sender import MicrophoneSender
from hume.legacy._voice.voice_socket import VoiceSocket

logger = logging.getLogger(__name__)


@dataclass
class MicrophoneInterface:
    """Interface for connecting a device microphone to an EVI connection."""

    DEFAULT_ALLOW_USER_INTERRUPT: ClassVar[bool] = False

    @classmethod
    async def start(
        cls,
        socket: VoiceSocket,
        device: int | None = Microphone.DEFAULT_DEVICE,
        allow_user_interrupt: bool = DEFAULT_ALLOW_USER_INTERRUPT,
    ) -> None:
        """Start the microphone interface.

        Args:
            socket (VoiceSocket): EVI socket.
            device (int | None): Device index for the microphone.
            allow_user_interrupt (bool): Whether to allow the user to interrupt EVI.
        """
        with Microphone.context(device=device) as microphone:
            sender = MicrophoneSender.new(microphone=microphone, allow_interrupt=allow_user_interrupt)
            chat_client = ChatClient.new(sender=sender)
            print("Configuring socket with microphone settings...")
            await socket.update_session_settings(
                sample_rate=microphone.sample_rate,
                num_channels=microphone.num_channels,
            )
            print("Microphone connected. Say something!")
            await chat_client.run(socket=socket)



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...core.request_options import RequestOptions
from ..types.return_paged_custom_voices import ReturnPagedCustomVoices
from ...core.pydantic_utilities import parse_obj_as
from ..errors.bad_request_error import BadRequestError
from ..types.error_response import ErrorResponse
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ..types.posted_custom_voice_base_voice import PostedCustomVoiceBaseVoice
from ..types.posted_custom_voice_parameters import PostedCustomVoiceParameters
from ..types.return_custom_voice import ReturnCustomVoice
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.jsonable_encoder import jsonable_encoder
from ...core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomVoicesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_custom_voices(
        self,
        *,
        page_number: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ReturnPagedCustomVoices:
        """
        Fetches a paginated list of **Custom Voices**.

        Refer to our [voices guide](/docs/empathic-voice-interface-evi/voices) for details on creating a custom voice.

        Parameters
        ----------
        page_number : typing.Optional[int]
            Specifies the page number to retrieve, enabling pagination.

            This parameter uses zero-based indexing. For example, setting `page_number` to 0 retrieves the first page of results (items 0-9 if `page_size` is 10), setting `page_number` to 1 retrieves the second page (items 10-19), and so on. Defaults to 0, which retrieves the first page.

        page_size : typing.Optional[int]
            Specifies the maximum number of results to include per page, enabling pagination. The value must be between 1 and 100, inclusive.

            For example, if `page_size` is set to 10, each page will include up to 10 items. Defaults to 10.

        name : typing.Optional[str]
            Filter to only include custom voices with name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ReturnPagedCustomVoices
            Success

        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.empathic_voice.custom_voices.list_custom_voices()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/evi/custom_voices",
            method="GET",
            params={
                "page_number": page_number,
                "page_size": page_size,
                "name": name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ReturnPagedCustomVoices,
                    parse_obj_as(
                        type_=ReturnPagedCustomVoices,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_custom_voice(
        self,
        *,
        name: str,
        base_voice: PostedCustomVoiceBaseVoice,
        parameters: typing.Optional[PostedCustomVoiceParameters] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ReturnCustomVoice:
        """
        Creates a **Custom Voice** that can be added to an [EVI configuration](/reference/empathic-voice-interface-evi/configs/create-config).

        Refer to our [voices guide](/docs/empathic-voice-interface-evi/voices) for details on creating a custom voice.

        Parameters
        ----------
        name : str
            The name of the Custom Voice. Maximum length of 75 characters. Will be converted to all-uppercase. (e.g., "sample voice" becomes "SAMPLE VOICE")

        base_voice : PostedCustomVoiceBaseVoice
            Specifies the base voice used to create the Custom Voice.

        parameters : typing.Optional[PostedCustomVoiceParameters]
            The specified attributes of a Custom Voice.

            If no parameters are specified then all attributes will be set to their defaults, meaning no modfications will be made to the base voice.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ReturnCustomVoice
            Created

        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.empathic_voice.custom_voices.create_custom_voice(
            name="name",
            base_voice="ITO",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/evi/custom_voices",
            method="POST",
            json={
                "name": name,
                "base_voice": base_voice,
                "parameters": convert_and_respect_annotation_metadata(
                    object_=parameters, annotation=PostedCustomVoiceParameters, direction="write"
                ),
                "parameter_model": "20241004-11parameter",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ReturnCustomVoice,
                    parse_obj_as(
                        type_=ReturnCustomVoice,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_custom_voice(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ReturnCustomVoice:
        """
        Fetches a specific **Custom Voice** by ID.

        Refer to our [voices guide](/docs/empathic-voice-interface-evi/voices) for details on creating a custom voice.

        Parameters
        ----------
        id : str
            Identifier for a Custom Voice. Formatted as a UUID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ReturnCustomVoice
            Success

        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.empathic_voice.custom_voices.get_custom_voice(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/custom_voices/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ReturnCustomVoice,
                    parse_obj_as(
                        type_=ReturnCustomVoice,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_custom_voice_version(
        self,
        id: str,
        *,
        name: str,
        base_voice: PostedCustomVoiceBaseVoice,
        parameters: typing.Optional[PostedCustomVoiceParameters] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ReturnCustomVoice:
        """
        Updates a **Custom Voice** by creating a new version of the **Custom Voice**.

        Refer to our [voices guide](/docs/empathic-voice-interface-evi/voices) for details on creating a custom voice.

        Parameters
        ----------
        id : str
            Identifier for a Custom Voice. Formatted as a UUID.

        name : str
            The name of the Custom Voice. Maximum length of 75 characters. Will be converted to all-uppercase. (e.g., "sample voice" becomes "SAMPLE VOICE")

        base_voice : PostedCustomVoiceBaseVoice
            Specifies the base voice used to create the Custom Voice.

        parameters : typing.Optional[PostedCustomVoiceParameters]
            The specified attributes of a Custom Voice.

            If no parameters are specified then all attributes will be set to their defaults, meaning no modfications will be made to the base voice.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ReturnCustomVoice
            Created

        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.empathic_voice.custom_voices.create_custom_voice_version(
            id="id",
            name="name",
            base_voice="ITO",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/custom_voices/{jsonable_encoder(id)}",
            method="POST",
            json={
                "name": name,
                "base_voice": base_voice,
                "parameters": convert_and_respect_annotation_metadata(
                    object_=parameters, annotation=PostedCustomVoiceParameters, direction="write"
                ),
                "parameter_model": "20241004-11parameter",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ReturnCustomVoice,
                    parse_obj_as(
                        type_=ReturnCustomVoice,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_custom_voice(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deletes a **Custom Voice** and its versions.

        Refer to our [voices guide](/docs/empathic-voice-interface-evi/voices) for details on creating a custom voice.

        Parameters
        ----------
        id : str
            Identifier for a Custom Voice. Formatted as a UUID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.empathic_voice.custom_voices.delete_custom_voice(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/custom_voices/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_custom_voice_name(
        self, id: str, *, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> str:
        """
        Updates the name of a **Custom Voice**.

        Refer to our [voices guide](/docs/empathic-voice-interface-evi/voices) for details on creating a custom voice.

        Parameters
        ----------
        id : str
            Identifier for a Custom Voice. Formatted as a UUID.

        name : str
            The name of the Custom Voice. Maximum length of 75 characters. Will be converted to all-uppercase. (e.g., "sample voice" becomes "SAMPLE VOICE")

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/custom_voices/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "name": name,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return _response.text  # type: ignore
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncCustomVoicesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_custom_voices(
        self,
        *,
        page_number: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ReturnPagedCustomVoices:
        """
        Fetches a paginated list of **Custom Voices**.

        Refer to our [voices guide](/docs/empathic-voice-interface-evi/voices) for details on creating a custom voice.

        Parameters
        ----------
        page_number : typing.Optional[int]
            Specifies the page number to retrieve, enabling pagination.

            This parameter uses zero-based indexing. For example, setting `page_number` to 0 retrieves the first page of results (items 0-9 if `page_size` is 10), setting `page_number` to 1 retrieves the second page (items 10-19), and so on. Defaults to 0, which retrieves the first page.

        page_size : typing.Optional[int]
            Specifies the maximum number of results to include per page, enabling pagination. The value must be between 1 and 100, inclusive.

            For example, if `page_size` is set to 10, each page will include up to 10 items. Defaults to 10.

        name : typing.Optional[str]
            Filter to only include custom voices with name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ReturnPagedCustomVoices
            Success

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.empathic_voice.custom_voices.list_custom_voices()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/evi/custom_voices",
            method="GET",
            params={
                "page_number": page_number,
                "page_size": page_size,
                "name": name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ReturnPagedCustomVoices,
                    parse_obj_as(
                        type_=ReturnPagedCustomVoices,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_custom_voice(
        self,
        *,
        name: str,
        base_voice: PostedCustomVoiceBaseVoice,
        parameters: typing.Optional[PostedCustomVoiceParameters] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ReturnCustomVoice:
        """
        Creates a **Custom Voice** that can be added to an [EVI configuration](/reference/empathic-voice-interface-evi/configs/create-config).

        Refer to our [voices guide](/docs/empathic-voice-interface-evi/voices) for details on creating a custom voice.

        Parameters
        ----------
        name : str
            The name of the Custom Voice. Maximum length of 75 characters. Will be converted to all-uppercase. (e.g., "sample voice" becomes "SAMPLE VOICE")

        base_voice : PostedCustomVoiceBaseVoice
            Specifies the base voice used to create the Custom Voice.

        parameters : typing.Optional[PostedCustomVoiceParameters]
            The specified attributes of a Custom Voice.

            If no parameters are specified then all attributes will be set to their defaults, meaning no modfications will be made to the base voice.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ReturnCustomVoice
            Created

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.empathic_voice.custom_voices.create_custom_voice(
                name="name",
                base_voice="ITO",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/evi/custom_voices",
            method="POST",
            json={
                "name": name,
                "base_voice": base_voice,
                "parameters": convert_and_respect_annotation_metadata(
                    object_=parameters, annotation=PostedCustomVoiceParameters, direction="write"
                ),
                "parameter_model": "20241004-11parameter",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ReturnCustomVoice,
                    parse_obj_as(
                        type_=ReturnCustomVoice,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_custom_voice(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ReturnCustomVoice:
        """
        Fetches a specific **Custom Voice** by ID.

        Refer to our [voices guide](/docs/empathic-voice-interface-evi/voices) for details on creating a custom voice.

        Parameters
        ----------
        id : str
            Identifier for a Custom Voice. Formatted as a UUID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ReturnCustomVoice
            Success

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.empathic_voice.custom_voices.get_custom_voice(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/custom_voices/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ReturnCustomVoice,
                    parse_obj_as(
                        type_=ReturnCustomVoice,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_custom_voice_version(
        self,
        id: str,
        *,
        name: str,
        base_voice: PostedCustomVoiceBaseVoice,
        parameters: typing.Optional[PostedCustomVoiceParameters] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ReturnCustomVoice:
        """
        Updates a **Custom Voice** by creating a new version of the **Custom Voice**.

        Refer to our [voices guide](/docs/empathic-voice-interface-evi/voices) for details on creating a custom voice.

        Parameters
        ----------
        id : str
            Identifier for a Custom Voice. Formatted as a UUID.

        name : str
            The name of the Custom Voice. Maximum length of 75 characters. Will be converted to all-uppercase. (e.g., "sample voice" becomes "SAMPLE VOICE")

        base_voice : PostedCustomVoiceBaseVoice
            Specifies the base voice used to create the Custom Voice.

        parameters : typing.Optional[PostedCustomVoiceParameters]
            The specified attributes of a Custom Voice.

            If no parameters are specified then all attributes will be set to their defaults, meaning no modfications will be made to the base voice.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ReturnCustomVoice
            Created

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.empathic_voice.custom_voices.create_custom_voice_version(
                id="id",
                name="name",
                base_voice="ITO",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/custom_voices/{jsonable_encoder(id)}",
            method="POST",
            json={
                "name": name,
                "base_voice": base_voice,
                "parameters": convert_and_respect_annotation_metadata(
                    object_=parameters, annotation=PostedCustomVoiceParameters, direction="write"
                ),
                "parameter_model": "20241004-11parameter",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ReturnCustomVoice,
                    parse_obj_as(
                        type_=ReturnCustomVoice,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_custom_voice(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deletes a **Custom Voice** and its versions.

        Refer to our [voices guide](/docs/empathic-voice-interface-evi/voices) for details on creating a custom voice.

        Parameters
        ----------
        id : str
            Identifier for a Custom Voice. Formatted as a UUID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.empathic_voice.custom_voices.delete_custom_voice(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/custom_voices/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_custom_voice_name(
        self, id: str, *, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> str:
        """
        Updates the name of a **Custom Voice**.

        Refer to our [voices guide](/docs/empathic-voice-interface-evi/voices) for details on creating a custom voice.

        Parameters
        ----------
        id : str
            Identifier for a Custom Voice. Formatted as a UUID.

        name : str
            The name of the Custom Voice. Maximum length of 75 characters. Will be converted to all-uppercase. (e.g., "sample voice" becomes "SAMPLE VOICE")

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/custom_voices/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "name": name,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return _response.text  # type: ignore
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ....core.pydantic_utilities import UniversalBaseModel
import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class CompletedInference(UniversalBaseModel):
    created_timestamp_ms: int = pydantic.Field()
    """
    When this job was created (Unix timestamp in milliseconds).
    """

    started_timestamp_ms: int = pydantic.Field()
    """
    When this job started (Unix timestamp in milliseconds).
    """

    ended_timestamp_ms: int = pydantic.Field()
    """
    When this job ended (Unix timestamp in milliseconds).
    """

    num_predictions: int = pydantic.Field()
    """
    The number of predictions that were generated by this job.
    """

    num_errors: int = pydantic.Field()
    """
    The number of errors that occurred while running this job.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ....core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .job_details import JobDetails
from ....core.pydantic_utilities import IS_PYDANTIC_V2


class StreamErrorMessage(UniversalBaseModel):
    """
    Error message
    """

    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message text.
    """

    code: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the error.
    """

    payload_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    If a payload ID was passed in the request, the same payload ID will be sent back in the response body.
    """

    job_details: typing.Optional[JobDetails] = pydantic.Field(default=None)
    """
    If the job_details flag was set in the request, details about the current streaming job will be returned in the response body.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .role import Role
import pydantic
import typing
from .tool_call_message import ToolCallMessage
from .chat_message_tool_result import ChatMessageToolResult
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ChatMessage(UniversalBaseModel):
    role: Role = pydantic.Field()
    """
    Role of who is providing the message.
    """

    content: typing.Optional[str] = pydantic.Field(default=None)
    """
    Transcript of the message.
    """

    tool_call: typing.Optional[ToolCallMessage] = pydantic.Field(default=None)
    """
    Function call name and arguments.
    """

    tool_result: typing.Optional[ChatMessageToolResult] = pydantic.Field(default=None)
    """
    Function call response from client.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



"""Empathic Voice Interface client."""

import logging

from hume.legacy._voice.mixins.chat_mixin import ChatMixin
from hume.legacy._voice.mixins.chats_mixin import ChatsMixin
from hume.legacy._voice.mixins.configs_mixin import ConfigsMixin
from hume.legacy._voice.mixins.tools_mixin import ToolsMixin

logger = logging.getLogger(__name__)


class HumeVoiceClient(ChatMixin, ChatsMixin, ConfigsMixin, ToolsMixin):
    """Empathic Voice Interface client."""



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .chat_message import ChatMessage
from .inference import Inference
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class AssistantMessage(UniversalBaseModel):
    """
    When provided, the output is an assistant message.
    """

    type: typing.Literal["assistant_message"] = pydantic.Field(default="assistant_message")
    """
    The type of message sent through the socket; for an Assistant Message, this must be `assistant_message`.
    
    This message contains both a transcript of the assistant’s response and the expression measurement predictions of the assistant’s audio output.
    """

    custom_session_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Used to manage conversational state, correlate frontend and backend data, and persist conversations across EVI sessions.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the assistant message. Allows the Assistant Message to be tracked and referenced.
    """

    message: ChatMessage = pydantic.Field()
    """
    Transcript of the message.
    """

    models: Inference = pydantic.Field()
    """
    Inference model results.
    """

    from_text: bool = pydantic.Field()
    """
    Indicates if this message was inserted into the conversation as text from an [Assistant Input message](/reference/empathic-voice-interface-evi/chat/chat#send.Assistant%20Input.text).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .alternative import Alternative
from .bcp_47_tag import Bcp47Tag
from .bounding_box import BoundingBox
from .burst_prediction import BurstPrediction
from .classification import Classification
from .completed_embedding_generation import CompletedEmbeddingGeneration
from .completed_inference import CompletedInference
from .completed_state import CompletedState
from .completed_tl_inference import CompletedTlInference
from .completed_training import CompletedTraining
from .custom_model import CustomModel
from .custom_model_id import CustomModelId
from .custom_model_prediction import CustomModelPrediction
from .custom_model_request import CustomModelRequest
from .custom_model_version_id import CustomModelVersionId
from .custom_models_inference_job import CustomModelsInferenceJob
from .custom_models_training_job import CustomModelsTrainingJob
from .dataset import Dataset
from .dataset_id import DatasetId
from .dataset_version_id import DatasetVersionId
from .descriptions_score import DescriptionsScore
from .direction import Direction
from .embedding_generation_base_request import EmbeddingGenerationBaseRequest
from .embedding_generation_job import EmbeddingGenerationJob
from .emotion_score import EmotionScore
from .error import Error
from .evaluation_args import EvaluationArgs
from .face import Face
from .face_prediction import FacePrediction
from .facemesh_prediction import FacemeshPrediction
from .facs_score import FacsScore
from .failed import Failed
from .failed_state import FailedState
from .file import File
from .granularity import Granularity
from .grouped_predictions_burst_prediction import GroupedPredictionsBurstPrediction
from .grouped_predictions_face_prediction import GroupedPredictionsFacePrediction
from .grouped_predictions_facemesh_prediction import GroupedPredictionsFacemeshPrediction
from .grouped_predictions_language_prediction import GroupedPredictionsLanguagePrediction
from .grouped_predictions_ner_prediction import GroupedPredictionsNerPrediction
from .grouped_predictions_prosody_prediction import GroupedPredictionsProsodyPrediction
from .in_progress import InProgress
from .in_progress_state import InProgressState
from .inference_base_request import InferenceBaseRequest
from .inference_job import InferenceJob
from .inference_prediction import InferencePrediction
from .inference_request import InferenceRequest
from .inference_results import InferenceResults
from .inference_source_predict_result import InferenceSourcePredictResult
from .job_embedding_generation import JobEmbeddingGeneration
from .job_id import JobId
from .job_inference import JobInference
from .job_tl_inference import JobTlInference
from .job_training import JobTraining
from .language import Language
from .language_prediction import LanguagePrediction
from .models import Models
from .models_predictions import ModelsPredictions
from .ner import Ner
from .ner_prediction import NerPrediction
from .null import Null
from .position_interval import PositionInterval
from .predictions_optional_null_burst_prediction import PredictionsOptionalNullBurstPrediction
from .predictions_optional_null_face_prediction import PredictionsOptionalNullFacePrediction
from .predictions_optional_null_facemesh_prediction import PredictionsOptionalNullFacemeshPrediction
from .predictions_optional_transcription_metadata_language_prediction import (
    PredictionsOptionalTranscriptionMetadataLanguagePrediction,
)
from .predictions_optional_transcription_metadata_ner_prediction import (
    PredictionsOptionalTranscriptionMetadataNerPrediction,
)
from .predictions_optional_transcription_metadata_prosody_prediction import (
    PredictionsOptionalTranscriptionMetadataProsodyPrediction,
)
from .prosody import Prosody
from .prosody_prediction import ProsodyPrediction
from .queued import Queued
from .queued_state import QueuedState
from .registry_file_detail import RegistryFileDetail
from .regression import Regression
from .sentiment_score import SentimentScore
from .sort_by import SortBy
from .source import Source
from .source_file import SourceFile
from .source_text_source import SourceTextSource
from .source_url import SourceUrl
from .state_embedding_generation import StateEmbeddingGeneration
from .state_embedding_generation_completed_embedding_generation import (
    StateEmbeddingGenerationCompletedEmbeddingGeneration,
)
from .state_embedding_generation_failed import StateEmbeddingGenerationFailed
from .state_embedding_generation_in_progress import StateEmbeddingGenerationInProgress
from .state_embedding_generation_queued import StateEmbeddingGenerationQueued
from .state_inference import StateInference
from .state_tl_inference import StateTlInference
from .state_tl_inference_completed_tl_inference import StateTlInferenceCompletedTlInference
from .state_tl_inference_failed import StateTlInferenceFailed
from .state_tl_inference_in_progress import StateTlInferenceInProgress
from .state_tl_inference_queued import StateTlInferenceQueued
from .state_training import StateTraining
from .state_training_completed_training import StateTrainingCompletedTraining
from .state_training_failed import StateTrainingFailed
from .state_training_in_progress import StateTrainingInProgress
from .state_training_queued import StateTrainingQueued
from .status import Status
from .tag import Tag
from .target import Target
from .task import Task
from .task_classification import TaskClassification
from .task_regression import TaskRegression
from .text_source import TextSource
from .time_interval import TimeInterval
from .tl_inference_base_request import TlInferenceBaseRequest
from .tl_inference_prediction import TlInferencePrediction
from .tl_inference_results import TlInferenceResults
from .tl_inference_source_predict_result import TlInferenceSourcePredictResult
from .toxicity_score import ToxicityScore
from .training_base_request import TrainingBaseRequest
from .training_custom_model import TrainingCustomModel
from .transcription import Transcription
from .transcription_metadata import TranscriptionMetadata
from .type import Type
from .unconfigurable import Unconfigurable
from .union_job import UnionJob
from .union_predict_result import UnionPredictResult
from .url import Url
from .validation_args import ValidationArgs
from .when import When
from .window import Window

__all__ = [
    "Alternative",
    "Bcp47Tag",
    "BoundingBox",
    "BurstPrediction",
    "Classification",
    "CompletedEmbeddingGeneration",
    "CompletedInference",
    "CompletedState",
    "CompletedTlInference",
    "CompletedTraining",
    "CustomModel",
    "CustomModelId",
    "CustomModelPrediction",
    "CustomModelRequest",
    "CustomModelVersionId",
    "CustomModelsInferenceJob",
    "CustomModelsTrainingJob",
    "Dataset",
    "DatasetId",
    "DatasetVersionId",
    "DescriptionsScore",
    "Direction",
    "EmbeddingGenerationBaseRequest",
    "EmbeddingGenerationJob",
    "EmotionScore",
    "Error",
    "EvaluationArgs",
    "Face",
    "FacePrediction",
    "FacemeshPrediction",
    "FacsScore",
    "Failed",
    "FailedState",
    "File",
    "Granularity",
    "GroupedPredictionsBurstPrediction",
    "GroupedPredictionsFacePrediction",
    "GroupedPredictionsFacemeshPrediction",
    "GroupedPredictionsLanguagePrediction",
    "GroupedPredictionsNerPrediction",
    "GroupedPredictionsProsodyPrediction",
    "InProgress",
    "InProgressState",
    "InferenceBaseRequest",
    "InferenceJob",
    "InferencePrediction",
    "InferenceRequest",
    "InferenceResults",
    "InferenceSourcePredictResult",
    "JobEmbeddingGeneration",
    "JobId",
    "JobInference",
    "JobTlInference",
    "JobTraining",
    "Language",
    "LanguagePrediction",
    "Models",
    "ModelsPredictions",
    "Ner",
    "NerPrediction",
    "Null",
    "PositionInterval",
    "PredictionsOptionalNullBurstPrediction",
    "PredictionsOptionalNullFacePrediction",
    "PredictionsOptionalNullFacemeshPrediction",
    "PredictionsOptionalTranscriptionMetadataLanguagePrediction",
    "PredictionsOptionalTranscriptionMetadataNerPrediction",
    "PredictionsOptionalTranscriptionMetadataProsodyPrediction",
    "Prosody",
    "ProsodyPrediction",
    "Queued",
    "QueuedState",
    "RegistryFileDetail",
    "Regression",
    "SentimentScore",
    "SortBy",
    "Source",
    "SourceFile",
    "SourceTextSource",
    "SourceUrl",
    "StateEmbeddingGeneration",
    "StateEmbeddingGenerationCompletedEmbeddingGeneration",
    "StateEmbeddingGenerationFailed",
    "StateEmbeddingGenerationInProgress",
    "StateEmbeddingGenerationQueued",
    "StateInference",
    "StateTlInference",
    "StateTlInferenceCompletedTlInference",
    "StateTlInferenceFailed",
    "StateTlInferenceInProgress",
    "StateTlInferenceQueued",
    "StateTraining",
    "StateTrainingCompletedTraining",
    "StateTrainingFailed",
    "StateTrainingInProgress",
    "StateTrainingQueued",
    "Status",
    "Tag",
    "Target",
    "Task",
    "TaskClassification",
    "TaskRegression",
    "TextSource",
    "TimeInterval",
    "TlInferenceBaseRequest",
    "TlInferencePrediction",
    "TlInferenceResults",
    "TlInferenceSourcePredictResult",
    "ToxicityScore",
    "TrainingBaseRequest",
    "TrainingCustomModel",
    "Transcription",
    "TranscriptionMetadata",
    "Type",
    "Unconfigurable",
    "UnionJob",
    "UnionPredictResult",
    "Url",
    "ValidationArgs",
    "When",
    "Window",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ReturnCustomVoiceParameters(UniversalBaseModel):
    """
    The specified attributes of a Custom Voice. If a parameter's value is `0` (default), it will not be included in the response.
    """

    gender: typing.Optional[int] = pydantic.Field(default=None)
    """
    The vocalization of gender, ranging between more masculine and more feminine.
    
    The default value is `0`, with a minimum of `-100` (more masculine) and a maximum of `100` (more feminine). A value of `0` leaves this parameter unchanged from the base voice.
    """

    articulation: typing.Optional[int] = pydantic.Field(default=None)
    """
    The clarity of the voice, ranging between mumbled and articulate.
    
    The default value is `0`, with a minimum of `-100` (mumbled) and a maximum of `100` (articulate). A value of `0` leaves this parameter unchanged from the base voice.
    """

    assertiveness: typing.Optional[int] = pydantic.Field(default=None)
    """
    The firmness of the voice, ranging between whiny and bold.
    
    The default value is `0`, with a minimum of `-100` (whiny) and a maximum of `100` (bold). A value of `0` leaves this parameter unchanged from the base voice.
    """

    buoyancy: typing.Optional[int] = pydantic.Field(default=None)
    """
    The density of the voice, ranging between deflated and buoyant.
    
    The default value is `0`, with a minimum of `-100` (deflated) and a maximum of `100` (buoyant). A value of `0` leaves this parameter unchanged from the base voice.
    """

    confidence: typing.Optional[int] = pydantic.Field(default=None)
    """
    The assuredness of the voice, ranging between shy and confident.
    
    The default value is `0`, with a minimum of `-100` (shy) and a maximum of `100` (confident). A value of `0` leaves this parameter unchanged from the base voice.
    """

    enthusiasm: typing.Optional[int] = pydantic.Field(default=None)
    """
    The excitement within the voice, ranging between calm and enthusiastic.
    
    The default value is `0`, with a minimum of `-100` (calm) and a maximum of `100` (enthusiastic). A value of `0` leaves this parameter unchanged from the base voice.
    """

    nasality: typing.Optional[int] = pydantic.Field(default=None)
    """
    The openness of the voice, ranging between clear and nasal.
    
    The default value is `0`, with a minimum of `-100` (clear) and a maximum of `100` (nasal). A value of `0` leaves this parameter unchanged from the base voice.
    """

    relaxedness: typing.Optional[int] = pydantic.Field(default=None)
    """
    The stress within the voice, ranging between tense and relaxed.
    
    The default value is `0`, with a minimum of `-100` (tense) and a maximum of `100` (relaxed). A value of `0` leaves this parameter unchanged from the base voice.
    """

    smoothness: typing.Optional[int] = pydantic.Field(default=None)
    """
    The texture of the voice, ranging between smooth and staccato.
    
    The default value is `0`, with a minimum of `-100` (smooth) and a maximum of `100` (staccato). A value of `0` leaves this parameter unchanged from the base voice.
    """

    tepidity: typing.Optional[int] = pydantic.Field(default=None)
    """
    The liveliness behind the voice, ranging between tepid and vigorous.
    
    The default value is `0`, with a minimum of `-100` (tepid) and a maximum of `100` (vigorous). A value of `0` leaves this parameter unchanged from the base voice.
    """

    tightness: typing.Optional[int] = pydantic.Field(default=None)
    """
    The containment of the voice, ranging between tight and breathy.
    
    The default value is `0`, with a minimum of `-100` (tight) and a maximum of `100` (breathy). A value of `0` leaves this parameter unchanged from the base voice.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



"""Utilities for audio playback."""

import asyncio
from io import BytesIO
from hume.legacy.error.hume_client_exception import HumeClientException

try:
    import pydub.playback
    from pydub import AudioSegment
    HAS_AUDIO_DEPENDENCIES = True
except ModuleNotFoundError:
    HAS_AUDIO_DEPENDENCIES = False


# NOTE:
# - expects that byte_str is a valid audio file with the appropriate headers
# - implicitly relies on simpleaudio:
#   - [https://stackoverflow.com/a/20746883]
#   - [https://github.com/jiaaro/pydub#playback]
#   - [https://github.com/jiaaro/pydub/blob/master/pydub/playback.py]
async def play_audio(byte_str: bytes) -> None:
    """Play a byte string of audio data with the system audio output device.

    Args:
        byte_str (bytes): Byte string of audio data.
    """
    if not HAS_AUDIO_DEPENDENCIES:
        raise HumeClientException(
            'Run `pip install "hume[legacy]"` to install dependencies required to use audio playback.'
        )
    segment = AudioSegment.from_file(BytesIO(byte_str))
    await asyncio.to_thread(pydub.playback.play, segment)



"""Session settings model."""

from __future__ import annotations

from pydantic import BaseModel


class AudioSettings(BaseModel):
    """Audio settings model."""

    encoding: str | None = "linear16"
    channels: int | None = None
    sample_rate: int | None = None


class SessionSettings(BaseModel):
    """Session settings model."""

    type: str = "session_settings"

    system_prompt: str | None = None
    audio: AudioSettings | None = None



# This file was auto-generated by Fern from our API Definition.

from .assistant_end import AssistantEnd
from .assistant_input import AssistantInput
from .assistant_message import AssistantMessage
from .audio_configuration import AudioConfiguration
from .audio_input import AudioInput
from .audio_output import AudioOutput
from .built_in_tool import BuiltInTool
from .builtin_tool_config import BuiltinToolConfig
from .chat_message import ChatMessage
from .chat_message_tool_result import ChatMessageToolResult
from .chat_metadata import ChatMetadata
from .context import Context
from .context_type import ContextType
from .emotion_scores import EmotionScores
from .encoding import Encoding
from .error_level import ErrorLevel
from .error_response import ErrorResponse
from .function_call_response_input import FunctionCallResponseInput
from .http_validation_error import HttpValidationError
from .inference import Inference
from .json_message import JsonMessage
from .millisecond_interval import MillisecondInterval
from .pause_assistant_message import PauseAssistantMessage
from .posted_builtin_tool import PostedBuiltinTool
from .posted_builtin_tool_name import PostedBuiltinToolName
from .posted_config_prompt_spec import PostedConfigPromptSpec
from .posted_custom_voice import PostedCustomVoice
from .posted_custom_voice_base_voice import PostedCustomVoiceBaseVoice
from .posted_custom_voice_parameters import PostedCustomVoiceParameters
from .posted_ellm_model import PostedEllmModel
from .posted_event_message_spec import PostedEventMessageSpec
from .posted_event_message_specs import PostedEventMessageSpecs
from .posted_language_model import PostedLanguageModel
from .posted_language_model_model_provider import PostedLanguageModelModelProvider
from .posted_prompt_spec import PostedPromptSpec
from .posted_timeout_spec import PostedTimeoutSpec
from .posted_timeout_specs import PostedTimeoutSpecs
from .posted_timeout_specs_inactivity import PostedTimeoutSpecsInactivity
from .posted_timeout_specs_max_duration import PostedTimeoutSpecsMaxDuration
from .posted_user_defined_tool_spec import PostedUserDefinedToolSpec
from .posted_voice import PostedVoice
from .prosody_inference import ProsodyInference
from .resume_assistant_message import ResumeAssistantMessage
from .return_active_chat_count import ReturnActiveChatCount
from .return_active_chat_count_per_tag import ReturnActiveChatCountPerTag
from .return_builtin_tool import ReturnBuiltinTool
from .return_builtin_tool_tool_type import ReturnBuiltinToolToolType
from .return_chat import ReturnChat
from .return_chat_event import ReturnChatEvent
from .return_chat_event_role import ReturnChatEventRole
from .return_chat_event_type import ReturnChatEventType
from .return_chat_group import ReturnChatGroup
from .return_chat_group_paged_chats import ReturnChatGroupPagedChats
from .return_chat_group_paged_chats_pagination_direction import ReturnChatGroupPagedChatsPaginationDirection
from .return_chat_group_paged_events import ReturnChatGroupPagedEvents
from .return_chat_group_paged_events_pagination_direction import ReturnChatGroupPagedEventsPaginationDirection
from .return_chat_paged_events import ReturnChatPagedEvents
from .return_chat_paged_events_pagination_direction import ReturnChatPagedEventsPaginationDirection
from .return_chat_paged_events_status import ReturnChatPagedEventsStatus
from .return_chat_status import ReturnChatStatus
from .return_config import ReturnConfig
from .return_config_spec import ReturnConfigSpec
from .return_custom_voice import ReturnCustomVoice
from .return_custom_voice_base_voice import ReturnCustomVoiceBaseVoice
from .return_custom_voice_parameters import ReturnCustomVoiceParameters
from .return_ellm_model import ReturnEllmModel
from .return_event_message_spec import ReturnEventMessageSpec
from .return_event_message_specs import ReturnEventMessageSpecs
from .return_language_model import ReturnLanguageModel
from .return_language_model_model_provider import ReturnLanguageModelModelProvider
from .return_paged_chat_groups import ReturnPagedChatGroups
from .return_paged_chat_groups_pagination_direction import ReturnPagedChatGroupsPaginationDirection
from .return_paged_chats import ReturnPagedChats
from .return_paged_chats_pagination_direction import ReturnPagedChatsPaginationDirection
from .return_paged_configs import ReturnPagedConfigs
from .return_paged_custom_voices import ReturnPagedCustomVoices
from .return_paged_prompts import ReturnPagedPrompts
from .return_paged_user_defined_tools import ReturnPagedUserDefinedTools
from .return_prompt import ReturnPrompt
from .return_prompt_version_type import ReturnPromptVersionType
from .return_timeout_spec import ReturnTimeoutSpec
from .return_timeout_specs import ReturnTimeoutSpecs
from .return_user_defined_tool import ReturnUserDefinedTool
from .return_user_defined_tool_tool_type import ReturnUserDefinedToolToolType
from .return_user_defined_tool_version_type import ReturnUserDefinedToolVersionType
from .return_voice import ReturnVoice
from .role import Role
from .session_settings import SessionSettings
from .text_input import TextInput
from .tool import Tool
from .tool_call_message import ToolCallMessage
from .tool_error_message import ToolErrorMessage
from .tool_response_message import ToolResponseMessage
from .tool_type import ToolType
from .tts_input import TtsInput
from .user_input import UserInput
from .user_interruption import UserInterruption
from .user_message import UserMessage
from .validation_error import ValidationError
from .validation_error_loc_item import ValidationErrorLocItem
from .voice_name_enum import VoiceNameEnum
from .web_socket_error import WebSocketError

__all__ = [
    "AssistantEnd",
    "AssistantInput",
    "AssistantMessage",
    "AudioConfiguration",
    "AudioInput",
    "AudioOutput",
    "BuiltInTool",
    "BuiltinToolConfig",
    "ChatMessage",
    "ChatMessageToolResult",
    "ChatMetadata",
    "Context",
    "ContextType",
    "EmotionScores",
    "Encoding",
    "ErrorLevel",
    "ErrorResponse",
    "FunctionCallResponseInput",
    "HttpValidationError",
    "Inference",
    "JsonMessage",
    "MillisecondInterval",
    "PauseAssistantMessage",
    "PostedBuiltinTool",
    "PostedBuiltinToolName",
    "PostedConfigPromptSpec",
    "PostedCustomVoice",
    "PostedCustomVoiceBaseVoice",
    "PostedCustomVoiceParameters",
    "PostedEllmModel",
    "PostedEventMessageSpec",
    "PostedEventMessageSpecs",
    "PostedLanguageModel",
    "PostedLanguageModelModelProvider",
    "PostedPromptSpec",
    "PostedTimeoutSpec",
    "PostedTimeoutSpecs",
    "PostedTimeoutSpecsInactivity",
    "PostedTimeoutSpecsMaxDuration",
    "PostedUserDefinedToolSpec",
    "PostedVoice",
    "ProsodyInference",
    "ResumeAssistantMessage",
    "ReturnActiveChatCount",
    "ReturnActiveChatCountPerTag",
    "ReturnBuiltinTool",
    "ReturnBuiltinToolToolType",
    "ReturnChat",
    "ReturnChatEvent",
    "ReturnChatEventRole",
    "ReturnChatEventType",
    "ReturnChatGroup",
    "ReturnChatGroupPagedChats",
    "ReturnChatGroupPagedChatsPaginationDirection",
    "ReturnChatGroupPagedEvents",
    "ReturnChatGroupPagedEventsPaginationDirection",
    "ReturnChatPagedEvents",
    "ReturnChatPagedEventsPaginationDirection",
    "ReturnChatPagedEventsStatus",
    "ReturnChatStatus",
    "ReturnConfig",
    "ReturnConfigSpec",
    "ReturnCustomVoice",
    "ReturnCustomVoiceBaseVoice",
    "ReturnCustomVoiceParameters",
    "ReturnEllmModel",
    "ReturnEventMessageSpec",
    "ReturnEventMessageSpecs",
    "ReturnLanguageModel",
    "ReturnLanguageModelModelProvider",
    "ReturnPagedChatGroups",
    "ReturnPagedChatGroupsPaginationDirection",
    "ReturnPagedChats",
    "ReturnPagedChatsPaginationDirection",
    "ReturnPagedConfigs",
    "ReturnPagedCustomVoices",
    "ReturnPagedPrompts",
    "ReturnPagedUserDefinedTools",
    "ReturnPrompt",
    "ReturnPromptVersionType",
    "ReturnTimeoutSpec",
    "ReturnTimeoutSpecs",
    "ReturnUserDefinedTool",
    "ReturnUserDefinedToolToolType",
    "ReturnUserDefinedToolVersionType",
    "ReturnVoice",
    "Role",
    "SessionSettings",
    "TextInput",
    "Tool",
    "ToolCallMessage",
    "ToolErrorMessage",
    "ToolResponseMessage",
    "ToolType",
    "TtsInput",
    "UserInput",
    "UserInterruption",
    "UserMessage",
    "ValidationError",
    "ValidationErrorLocItem",
    "VoiceNameEnum",
    "WebSocketError",
]



"""API request and response models for EVI tools."""

from __future__ import annotations

from hume.legacy._common.utilities.model_utilities import BaseModel


class ToolResponse(BaseModel):
    """Response model for an EVI tools."""

    id: str
    tool_type: str
    version: int
    version_description: str | None
    name: str
    created_on: int
    modified_on: int
    fallback_content: str | None
    description: str | None
    parameters: str


class ToolsResponse(BaseModel):
    """Response model for a page of EVI tools."""

    page_number: int
    page_size: int
    tools_page: list[ToolResponse]


class ToolMeta(BaseModel):
    """Tool metadata."""

    id: str
    version: int


class PostToolRequest(BaseModel):
    """Post request model for creating a new EVI tool."""

    name: str
    version_description: str | None
    description: str | None
    parameters: str
    fallback_content: str | None


class VoiceTool(BaseModel):
    """EVI tool."""

    id: str
    name: str
    created_on: int
    modified_on: int
    parameters: str
    description: str | None
    fallback_content: str | None



# This file was auto-generated by Fern from our API Definition.

from ....core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .text_position import TextPosition
from .emotion_embedding import EmotionEmbedding
from .sentiment import Sentiment
from .toxicity import Toxicity
from ....core.pydantic_utilities import IS_PYDANTIC_V2


class StreamModelPredictionsLanguagePredictionsItem(UniversalBaseModel):
    text: typing.Optional[str] = pydantic.Field(default=None)
    """
    A segment of text (like a word or a sentence).
    """

    position: typing.Optional[TextPosition] = None
    emotions: typing.Optional[EmotionEmbedding] = None
    sentiment: typing.Optional[Sentiment] = None
    toxicity: typing.Optional[Toxicity] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ....core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2


class JobDetails(UniversalBaseModel):
    """
    If the job_details flag was set in the request, details about the current streaming job will be returned in the response body.
    """

    job_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the current streaming job.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from .return_chat_paged_events_status import ReturnChatPagedEventsStatus
import typing
from .return_chat_paged_events_pagination_direction import ReturnChatPagedEventsPaginationDirection
from .return_chat_event import ReturnChatEvent
from .return_config_spec import ReturnConfigSpec
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ReturnChatPagedEvents(UniversalBaseModel):
    """
    A description of chat status with a paginated list of chat events returned from the server
    """

    id: str = pydantic.Field()
    """
    Identifier for a Chat. Formatted as a UUID.
    """

    chat_group_id: str = pydantic.Field()
    """
    Identifier for the Chat Group. Any chat resumed from this Chat will have the same `chat_group_id`. Formatted as a UUID.
    """

    status: ReturnChatPagedEventsStatus = pydantic.Field()
    """
    Indicates the current state of the chat. There are six possible statuses:
    
    - `ACTIVE`: The chat is currently active and ongoing.
    
    - `USER_ENDED`: The chat was manually ended by the user.
    
    - `USER_TIMEOUT`: The chat ended due to a user-defined timeout.
    
    - `MAX_DURATION_TIMEOUT`: The chat ended because it reached the maximum allowed duration.
    
    - `INACTIVITY_TIMEOUT`: The chat ended due to an inactivity timeout.
    
    - `ERROR`: The chat ended unexpectedly due to an error.
    """

    start_timestamp: int = pydantic.Field()
    """
    Time at which the Chat started. Measured in seconds since the Unix epoch.
    """

    end_timestamp: typing.Optional[int] = pydantic.Field(default=None)
    """
    Time at which the Chat ended. Measured in seconds since the Unix epoch.
    """

    pagination_direction: ReturnChatPagedEventsPaginationDirection = pydantic.Field()
    """
    Indicates the order in which the paginated results are presented, based on their creation date.
    
    It shows `ASC` for ascending order (chronological, with the oldest records first) or `DESC` for descending order (reverse-chronological, with the newest records first). This value corresponds to the `ascending_order` query parameter used in the request.
    """

    events_page: typing.List[ReturnChatEvent] = pydantic.Field()
    """
    List of Chat Events for the specified `page_number` and `page_size`.
    """

    metadata: typing.Optional[str] = pydantic.Field(default=None)
    """
    Stringified JSON with additional metadata about the chat.
    """

    page_number: int = pydantic.Field()
    """
    The page number of the returned list.
    
    This value corresponds to the `page_number` parameter specified in the request. Pagination uses zero-based indexing.
    """

    page_size: int = pydantic.Field()
    """
    The maximum number of items returned per page.
    
    This value corresponds to the `page_size` parameter specified in the request.
    """

    total_pages: int = pydantic.Field()
    """
    The total number of pages in the collection.
    """

    config: typing.Optional[ReturnConfigSpec] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from . import empathic_voice, expression_measurement
from .client import AsyncHumeClient, HumeClient
from .empathic_voice.chat.audio.asyncio_utilities import Stream
from .empathic_voice.chat.audio.microphone_interface import MicrophoneInterface
from .environment import HumeClientEnvironment
from .version import __version__

__all__ = [
    "AsyncHumeClient",
    "HumeClient",
    "HumeClientEnvironment",
    "MicrophoneInterface",
    "Stream",
    "__version__",
    "empathic_voice",
    "expression_measurement",
]



# This file was auto-generated by Fern from our API Definition.

import typing

VoiceNameEnum = typing.Union[
    typing.Literal[
        "ITO",
        "KORA",
        "DACHER",
        "AURA",
        "FINN",
        "SIENNA",
        "WILLOW",
        "SCOUT",
        "WHIMSY",
        "ACE",
        "JUNO",
        "STELLA",
        "HIRO",
        "SUNNY",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ..types.return_user_defined_tool import ReturnUserDefinedTool
from ..types.return_paged_user_defined_tools import ReturnPagedUserDefinedTools
from ...core.pydantic_utilities import parse_obj_as
from ..errors.bad_request_error import BadRequestError
from ..types.error_response import ErrorResponse
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...core.jsonable_encoder import jsonable_encoder
from ...core.client_wrapper import AsyncClientWrapper
from ...core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ToolsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_tools(
        self,
        *,
        page_number: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        restrict_to_most_recent: typing.Optional[bool] = None,
        name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[typing.Optional[ReturnUserDefinedTool]]:
        """
        Fetches a paginated list of **Tools**.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        page_number : typing.Optional[int]
            Specifies the page number to retrieve, enabling pagination.

            This parameter uses zero-based indexing. For example, setting `page_number` to 0 retrieves the first page of results (items 0-9 if `page_size` is 10), setting `page_number` to 1 retrieves the second page (items 10-19), and so on. Defaults to 0, which retrieves the first page.

        page_size : typing.Optional[int]
            Specifies the maximum number of results to include per page, enabling pagination. The value must be between 1 and 100, inclusive.

            For example, if `page_size` is set to 10, each page will include up to 10 items. Defaults to 10.

        restrict_to_most_recent : typing.Optional[bool]
            By default, `restrict_to_most_recent` is set to true, returning only the latest version of each tool. To include all versions of each tool in the list, set `restrict_to_most_recent` to false.

        name : typing.Optional[str]
            Filter to only include tools with name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[typing.Optional[ReturnUserDefinedTool]]
            Success

        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        response = client.empathic_voice.tools.list_tools(
            page_number=0,
            page_size=2,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        page_number = page_number if page_number is not None else 1
        _response = self._client_wrapper.httpx_client.request(
            "v0/evi/tools",
            method="GET",
            params={
                "page_number": page_number,
                "page_size": page_size,
                "restrict_to_most_recent": restrict_to_most_recent,
                "name": name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ReturnPagedUserDefinedTools,
                    parse_obj_as(
                        type_=ReturnPagedUserDefinedTools,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _has_next = True
                _get_next = lambda: self.list_tools(
                    page_number=page_number + 1,
                    page_size=page_size,
                    restrict_to_most_recent=restrict_to_most_recent,
                    name=name,
                    request_options=request_options,
                )
                _items = _parsed_response.tools_page
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_tool(
        self,
        *,
        name: str,
        parameters: str,
        version_description: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        fallback_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[ReturnUserDefinedTool]:
        """
        Creates a **Tool** that can be added to an [EVI configuration](/reference/empathic-voice-interface-evi/configs/create-config).

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        name : str
            Name applied to all versions of a particular Tool.

        parameters : str
            Stringified JSON defining the parameters used by this version of the Tool.

            These parameters define the inputs needed for the Tool’s execution, including the expected data type and description for each input field. Structured as a stringified JSON schema, this format ensures the Tool receives data in the expected format.

        version_description : typing.Optional[str]
            An optional description of the Tool version.

        description : typing.Optional[str]
            An optional description of what the Tool does, used by the supplemental LLM to choose when and how to call the function.

        fallback_content : typing.Optional[str]
            Optional text passed to the supplemental LLM in place of the tool call result. The LLM then uses this text to generate a response back to the user, ensuring continuity in the conversation if the Tool errors.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[ReturnUserDefinedTool]
            Created

        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.empathic_voice.tools.create_tool(
            name="get_current_weather",
            parameters='{ "type": "object", "properties": { "location": { "type": "string", "description": "The city and state, e.g. San Francisco, CA" }, "format": { "type": "string", "enum": ["celsius", "fahrenheit"], "description": "The temperature unit to use. Infer this from the users location." } }, "required": ["location", "format"] }',
            version_description="Fetches current weather and uses celsius or fahrenheit based on location of user.",
            description="This tool is for getting the current weather.",
            fallback_content="Unable to fetch current weather.",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/evi/tools",
            method="POST",
            json={
                "name": name,
                "version_description": version_description,
                "description": description,
                "parameters": parameters,
                "fallback_content": fallback_content,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[ReturnUserDefinedTool],
                    parse_obj_as(
                        type_=typing.Optional[ReturnUserDefinedTool],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list_tool_versions(
        self,
        id: str,
        *,
        page_number: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        restrict_to_most_recent: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ReturnPagedUserDefinedTools:
        """
        Fetches a list of a **Tool's** versions.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        id : str
            Identifier for a Tool. Formatted as a UUID.

        page_number : typing.Optional[int]
            Specifies the page number to retrieve, enabling pagination.

            This parameter uses zero-based indexing. For example, setting `page_number` to 0 retrieves the first page of results (items 0-9 if `page_size` is 10), setting `page_number` to 1 retrieves the second page (items 10-19), and so on. Defaults to 0, which retrieves the first page.

        page_size : typing.Optional[int]
            Specifies the maximum number of results to include per page, enabling pagination. The value must be between 1 and 100, inclusive.

            For example, if `page_size` is set to 10, each page will include up to 10 items. Defaults to 10.

        restrict_to_most_recent : typing.Optional[bool]
            By default, `restrict_to_most_recent` is set to true, returning only the latest version of each tool. To include all versions of each tool in the list, set `restrict_to_most_recent` to false.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ReturnPagedUserDefinedTools
            Success

        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.empathic_voice.tools.list_tool_versions(
            id="00183a3f-79ba-413d-9f3b-609864268bea",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/tools/{jsonable_encoder(id)}",
            method="GET",
            params={
                "page_number": page_number,
                "page_size": page_size,
                "restrict_to_most_recent": restrict_to_most_recent,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ReturnPagedUserDefinedTools,
                    parse_obj_as(
                        type_=ReturnPagedUserDefinedTools,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_tool_version(
        self,
        id: str,
        *,
        parameters: str,
        version_description: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        fallback_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[ReturnUserDefinedTool]:
        """
        Updates a **Tool** by creating a new version of the **Tool**.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        id : str
            Identifier for a Tool. Formatted as a UUID.

        parameters : str
            Stringified JSON defining the parameters used by this version of the Tool.

            These parameters define the inputs needed for the Tool’s execution, including the expected data type and description for each input field. Structured as a stringified JSON schema, this format ensures the Tool receives data in the expected format.

        version_description : typing.Optional[str]
            An optional description of the Tool version.

        description : typing.Optional[str]
            An optional description of what the Tool does, used by the supplemental LLM to choose when and how to call the function.

        fallback_content : typing.Optional[str]
            Optional text passed to the supplemental LLM in place of the tool call result. The LLM then uses this text to generate a response back to the user, ensuring continuity in the conversation if the Tool errors.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[ReturnUserDefinedTool]
            Created

        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.empathic_voice.tools.create_tool_version(
            id="00183a3f-79ba-413d-9f3b-609864268bea",
            parameters='{ "type": "object", "properties": { "location": { "type": "string", "description": "The city and state, e.g. San Francisco, CA" }, "format": { "type": "string", "enum": ["celsius", "fahrenheit", "kelvin"], "description": "The temperature unit to use. Infer this from the users location." } }, "required": ["location", "format"] }',
            version_description="Fetches current weather and uses celsius, fahrenheit, or kelvin based on location of user.",
            fallback_content="Unable to fetch current weather.",
            description="This tool is for getting the current weather.",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/tools/{jsonable_encoder(id)}",
            method="POST",
            json={
                "version_description": version_description,
                "description": description,
                "parameters": parameters,
                "fallback_content": fallback_content,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[ReturnUserDefinedTool],
                    parse_obj_as(
                        type_=typing.Optional[ReturnUserDefinedTool],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_tool(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deletes a **Tool** and its versions.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        id : str
            Identifier for a Tool. Formatted as a UUID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.empathic_voice.tools.delete_tool(
            id="00183a3f-79ba-413d-9f3b-609864268bea",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/tools/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_tool_name(self, id: str, *, name: str, request_options: typing.Optional[RequestOptions] = None) -> str:
        """
        Updates the name of a **Tool**.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        id : str
            Identifier for a Tool. Formatted as a UUID.

        name : str
            Name applied to all versions of a particular Tool.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            Success

        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.empathic_voice.tools.update_tool_name(
            id="00183a3f-79ba-413d-9f3b-609864268bea",
            name="get_current_temperature",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/tools/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "name": name,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return _response.text  # type: ignore
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_tool_version(
        self, id: str, version: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[ReturnUserDefinedTool]:
        """
        Fetches a specified version of a **Tool**.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        id : str
            Identifier for a Tool. Formatted as a UUID.

        version : int
            Version number for a Tool.

            Tools, Configs, Custom Voices, and Prompts are versioned. This versioning system supports iterative development, allowing you to progressively refine tools and revert to previous versions if needed.

            Version numbers are integer values representing different iterations of the Tool. Each update to the Tool increments its version number.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[ReturnUserDefinedTool]
            Success

        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.empathic_voice.tools.get_tool_version(
            id="00183a3f-79ba-413d-9f3b-609864268bea",
            version=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/tools/{jsonable_encoder(id)}/version/{jsonable_encoder(version)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[ReturnUserDefinedTool],
                    parse_obj_as(
                        type_=typing.Optional[ReturnUserDefinedTool],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_tool_version(
        self, id: str, version: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes a specified version of a **Tool**.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        id : str
            Identifier for a Tool. Formatted as a UUID.

        version : int
            Version number for a Tool.

            Tools, Configs, Custom Voices, and Prompts are versioned. This versioning system supports iterative development, allowing you to progressively refine tools and revert to previous versions if needed.

            Version numbers are integer values representing different iterations of the Tool. Each update to the Tool increments its version number.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.empathic_voice.tools.delete_tool_version(
            id="00183a3f-79ba-413d-9f3b-609864268bea",
            version=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/tools/{jsonable_encoder(id)}/version/{jsonable_encoder(version)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_tool_description(
        self,
        id: str,
        version: int,
        *,
        version_description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[ReturnUserDefinedTool]:
        """
        Updates the description of a specified **Tool** version.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        id : str
            Identifier for a Tool. Formatted as a UUID.

        version : int
            Version number for a Tool.

            Tools, Configs, Custom Voices, and Prompts are versioned. This versioning system supports iterative development, allowing you to progressively refine tools and revert to previous versions if needed.

            Version numbers are integer values representing different iterations of the Tool. Each update to the Tool increments its version number.

        version_description : typing.Optional[str]
            An optional description of the Tool version.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[ReturnUserDefinedTool]
            Success

        Examples
        --------
        from hume import HumeClient

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.empathic_voice.tools.update_tool_description(
            id="00183a3f-79ba-413d-9f3b-609864268bea",
            version=1,
            version_description="Fetches current temperature, precipitation, wind speed, AQI, and other weather conditions. Uses Celsius, Fahrenheit, or kelvin depending on user's region.",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/tools/{jsonable_encoder(id)}/version/{jsonable_encoder(version)}",
            method="PATCH",
            json={
                "version_description": version_description,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[ReturnUserDefinedTool],
                    parse_obj_as(
                        type_=typing.Optional[ReturnUserDefinedTool],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncToolsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_tools(
        self,
        *,
        page_number: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        restrict_to_most_recent: typing.Optional[bool] = None,
        name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[typing.Optional[ReturnUserDefinedTool]]:
        """
        Fetches a paginated list of **Tools**.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        page_number : typing.Optional[int]
            Specifies the page number to retrieve, enabling pagination.

            This parameter uses zero-based indexing. For example, setting `page_number` to 0 retrieves the first page of results (items 0-9 if `page_size` is 10), setting `page_number` to 1 retrieves the second page (items 10-19), and so on. Defaults to 0, which retrieves the first page.

        page_size : typing.Optional[int]
            Specifies the maximum number of results to include per page, enabling pagination. The value must be between 1 and 100, inclusive.

            For example, if `page_size` is set to 10, each page will include up to 10 items. Defaults to 10.

        restrict_to_most_recent : typing.Optional[bool]
            By default, `restrict_to_most_recent` is set to true, returning only the latest version of each tool. To include all versions of each tool in the list, set `restrict_to_most_recent` to false.

        name : typing.Optional[str]
            Filter to only include tools with name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[typing.Optional[ReturnUserDefinedTool]]
            Success

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            response = await client.empathic_voice.tools.list_tools(
                page_number=0,
                page_size=2,
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        page_number = page_number if page_number is not None else 1
        _response = await self._client_wrapper.httpx_client.request(
            "v0/evi/tools",
            method="GET",
            params={
                "page_number": page_number,
                "page_size": page_size,
                "restrict_to_most_recent": restrict_to_most_recent,
                "name": name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ReturnPagedUserDefinedTools,
                    parse_obj_as(
                        type_=ReturnPagedUserDefinedTools,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _has_next = True
                _get_next = lambda: self.list_tools(
                    page_number=page_number + 1,
                    page_size=page_size,
                    restrict_to_most_recent=restrict_to_most_recent,
                    name=name,
                    request_options=request_options,
                )
                _items = _parsed_response.tools_page
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_tool(
        self,
        *,
        name: str,
        parameters: str,
        version_description: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        fallback_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[ReturnUserDefinedTool]:
        """
        Creates a **Tool** that can be added to an [EVI configuration](/reference/empathic-voice-interface-evi/configs/create-config).

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        name : str
            Name applied to all versions of a particular Tool.

        parameters : str
            Stringified JSON defining the parameters used by this version of the Tool.

            These parameters define the inputs needed for the Tool’s execution, including the expected data type and description for each input field. Structured as a stringified JSON schema, this format ensures the Tool receives data in the expected format.

        version_description : typing.Optional[str]
            An optional description of the Tool version.

        description : typing.Optional[str]
            An optional description of what the Tool does, used by the supplemental LLM to choose when and how to call the function.

        fallback_content : typing.Optional[str]
            Optional text passed to the supplemental LLM in place of the tool call result. The LLM then uses this text to generate a response back to the user, ensuring continuity in the conversation if the Tool errors.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[ReturnUserDefinedTool]
            Created

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.empathic_voice.tools.create_tool(
                name="get_current_weather",
                parameters='{ "type": "object", "properties": { "location": { "type": "string", "description": "The city and state, e.g. San Francisco, CA" }, "format": { "type": "string", "enum": ["celsius", "fahrenheit"], "description": "The temperature unit to use. Infer this from the users location." } }, "required": ["location", "format"] }',
                version_description="Fetches current weather and uses celsius or fahrenheit based on location of user.",
                description="This tool is for getting the current weather.",
                fallback_content="Unable to fetch current weather.",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/evi/tools",
            method="POST",
            json={
                "name": name,
                "version_description": version_description,
                "description": description,
                "parameters": parameters,
                "fallback_content": fallback_content,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[ReturnUserDefinedTool],
                    parse_obj_as(
                        type_=typing.Optional[ReturnUserDefinedTool],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list_tool_versions(
        self,
        id: str,
        *,
        page_number: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        restrict_to_most_recent: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ReturnPagedUserDefinedTools:
        """
        Fetches a list of a **Tool's** versions.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        id : str
            Identifier for a Tool. Formatted as a UUID.

        page_number : typing.Optional[int]
            Specifies the page number to retrieve, enabling pagination.

            This parameter uses zero-based indexing. For example, setting `page_number` to 0 retrieves the first page of results (items 0-9 if `page_size` is 10), setting `page_number` to 1 retrieves the second page (items 10-19), and so on. Defaults to 0, which retrieves the first page.

        page_size : typing.Optional[int]
            Specifies the maximum number of results to include per page, enabling pagination. The value must be between 1 and 100, inclusive.

            For example, if `page_size` is set to 10, each page will include up to 10 items. Defaults to 10.

        restrict_to_most_recent : typing.Optional[bool]
            By default, `restrict_to_most_recent` is set to true, returning only the latest version of each tool. To include all versions of each tool in the list, set `restrict_to_most_recent` to false.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ReturnPagedUserDefinedTools
            Success

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.empathic_voice.tools.list_tool_versions(
                id="00183a3f-79ba-413d-9f3b-609864268bea",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/tools/{jsonable_encoder(id)}",
            method="GET",
            params={
                "page_number": page_number,
                "page_size": page_size,
                "restrict_to_most_recent": restrict_to_most_recent,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ReturnPagedUserDefinedTools,
                    parse_obj_as(
                        type_=ReturnPagedUserDefinedTools,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_tool_version(
        self,
        id: str,
        *,
        parameters: str,
        version_description: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        fallback_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[ReturnUserDefinedTool]:
        """
        Updates a **Tool** by creating a new version of the **Tool**.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        id : str
            Identifier for a Tool. Formatted as a UUID.

        parameters : str
            Stringified JSON defining the parameters used by this version of the Tool.

            These parameters define the inputs needed for the Tool’s execution, including the expected data type and description for each input field. Structured as a stringified JSON schema, this format ensures the Tool receives data in the expected format.

        version_description : typing.Optional[str]
            An optional description of the Tool version.

        description : typing.Optional[str]
            An optional description of what the Tool does, used by the supplemental LLM to choose when and how to call the function.

        fallback_content : typing.Optional[str]
            Optional text passed to the supplemental LLM in place of the tool call result. The LLM then uses this text to generate a response back to the user, ensuring continuity in the conversation if the Tool errors.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[ReturnUserDefinedTool]
            Created

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.empathic_voice.tools.create_tool_version(
                id="00183a3f-79ba-413d-9f3b-609864268bea",
                parameters='{ "type": "object", "properties": { "location": { "type": "string", "description": "The city and state, e.g. San Francisco, CA" }, "format": { "type": "string", "enum": ["celsius", "fahrenheit", "kelvin"], "description": "The temperature unit to use. Infer this from the users location." } }, "required": ["location", "format"] }',
                version_description="Fetches current weather and uses celsius, fahrenheit, or kelvin based on location of user.",
                fallback_content="Unable to fetch current weather.",
                description="This tool is for getting the current weather.",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/tools/{jsonable_encoder(id)}",
            method="POST",
            json={
                "version_description": version_description,
                "description": description,
                "parameters": parameters,
                "fallback_content": fallback_content,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[ReturnUserDefinedTool],
                    parse_obj_as(
                        type_=typing.Optional[ReturnUserDefinedTool],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_tool(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deletes a **Tool** and its versions.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        id : str
            Identifier for a Tool. Formatted as a UUID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.empathic_voice.tools.delete_tool(
                id="00183a3f-79ba-413d-9f3b-609864268bea",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/tools/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_tool_name(
        self, id: str, *, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> str:
        """
        Updates the name of a **Tool**.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        id : str
            Identifier for a Tool. Formatted as a UUID.

        name : str
            Name applied to all versions of a particular Tool.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            Success

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.empathic_voice.tools.update_tool_name(
                id="00183a3f-79ba-413d-9f3b-609864268bea",
                name="get_current_temperature",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/tools/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "name": name,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return _response.text  # type: ignore
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_tool_version(
        self, id: str, version: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[ReturnUserDefinedTool]:
        """
        Fetches a specified version of a **Tool**.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        id : str
            Identifier for a Tool. Formatted as a UUID.

        version : int
            Version number for a Tool.

            Tools, Configs, Custom Voices, and Prompts are versioned. This versioning system supports iterative development, allowing you to progressively refine tools and revert to previous versions if needed.

            Version numbers are integer values representing different iterations of the Tool. Each update to the Tool increments its version number.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[ReturnUserDefinedTool]
            Success

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.empathic_voice.tools.get_tool_version(
                id="00183a3f-79ba-413d-9f3b-609864268bea",
                version=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/tools/{jsonable_encoder(id)}/version/{jsonable_encoder(version)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[ReturnUserDefinedTool],
                    parse_obj_as(
                        type_=typing.Optional[ReturnUserDefinedTool],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_tool_version(
        self, id: str, version: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes a specified version of a **Tool**.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        id : str
            Identifier for a Tool. Formatted as a UUID.

        version : int
            Version number for a Tool.

            Tools, Configs, Custom Voices, and Prompts are versioned. This versioning system supports iterative development, allowing you to progressively refine tools and revert to previous versions if needed.

            Version numbers are integer values representing different iterations of the Tool. Each update to the Tool increments its version number.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.empathic_voice.tools.delete_tool_version(
                id="00183a3f-79ba-413d-9f3b-609864268bea",
                version=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/tools/{jsonable_encoder(id)}/version/{jsonable_encoder(version)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_tool_description(
        self,
        id: str,
        version: int,
        *,
        version_description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[ReturnUserDefinedTool]:
        """
        Updates the description of a specified **Tool** version.

        Refer to our [tool use](/docs/empathic-voice-interface-evi/tool-use#function-calling) guide for comprehensive instructions on defining and integrating tools into EVI.

        Parameters
        ----------
        id : str
            Identifier for a Tool. Formatted as a UUID.

        version : int
            Version number for a Tool.

            Tools, Configs, Custom Voices, and Prompts are versioned. This versioning system supports iterative development, allowing you to progressively refine tools and revert to previous versions if needed.

            Version numbers are integer values representing different iterations of the Tool. Each update to the Tool increments its version number.

        version_description : typing.Optional[str]
            An optional description of the Tool version.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[ReturnUserDefinedTool]
            Success

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.empathic_voice.tools.update_tool_description(
                id="00183a3f-79ba-413d-9f3b-609864268bea",
                version=1,
                version_description="Fetches current temperature, precipitation, wind speed, AQI, and other weather conditions. Uses Celsius, Fahrenheit, or kelvin depending on user's region.",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/tools/{jsonable_encoder(id)}/version/{jsonable_encoder(version)}",
            method="PATCH",
            json={
                "version_description": version_description,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[ReturnUserDefinedTool],
                    parse_obj_as(
                        type_=typing.Optional[ReturnUserDefinedTool],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class AssistantEnd(UniversalBaseModel):
    """
    When provided, the output is an assistant end message.
    """

    type: typing.Literal["assistant_end"] = pydantic.Field(default="assistant_end")
    """
    The type of message sent through the socket; for an Assistant End message, this must be `assistant_end`.
    
    This message indicates the conclusion of the assistant’s response, signaling that the assistant has finished speaking for the current conversational turn.
    """

    custom_session_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Used to manage conversational state, correlate frontend and backend data, and persist conversations across EVI sessions.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



"""API request and response models for EVI configurations."""

from __future__ import annotations

from pydantic import ConfigDict

from hume.legacy._common.utilities.model_utilities import BaseModel
from hume.legacy._voice.models.tools_models import ToolMeta


class PromptResponse(BaseModel):
    """Response model for an EVI prompt."""

    id: str
    version: int
    version_description: str | None
    name: str
    created_on: int
    modified_on: int
    text: str | None


class PromptsResponse(BaseModel):
    """Response model for a page of EVI prompts."""

    page_number: int
    page_size: int
    prompts: list[PromptResponse]


class PromptMeta(BaseModel):
    """Prompt metadata."""

    id: str
    version: int


class LanguageModelConfig(BaseModel):
    """Language model configuration for EVI."""

    model_provider: str
    model_resource: str
    temperature: float | None = None

    model_config = ConfigDict(protected_namespaces=())


class PostPromptRequest(BaseModel):
    """Post request model for creating a new EVI prompt."""

    name: str
    version_description: str | None
    text: str | None


class VoiceIdentityConfig(BaseModel):
    """Configuration for changing the voice of EVI."""

    provider: str
    name: str | None = None


class ConfigResponse(BaseModel):
    """Response model for an EVI configurations."""

    id: str
    version: int
    version_description: str | None
    name: str
    created_on: int
    modified_on: int
    prompt: PromptResponse | None
    voice: VoiceIdentityConfig | None


class ConfigsResponse(BaseModel):
    """Response model for a page of EVI configurations."""

    page_number: int
    page_size: int
    configs_page: list[ConfigResponse]


class BuiltinToolConfig(BaseModel):
    """Configuration for a built-in EVI tool."""

    name: str
    tool_type: str
    fallback_content: str | None


class PostConfigRequest(BaseModel):
    """Post request model for creating a new EVI configuration."""

    name: str
    version_description: str | None
    prompt: PromptMeta
    voice: VoiceIdentityConfig | None
    language_model: LanguageModelConfig | None
    tools: list[ToolMeta] | None


class ConfigMeta(BaseModel):
    """EVI configuration metadata."""

    id: str | None
    version: int | None


class VoiceConfig(BaseModel):
    """EVI configuration."""

    id: str
    name: str
    description: str | None
    created_on: int
    modified_on: int
    prompt: str | None
    voice: VoiceIdentityConfig | None



# This file was auto-generated by Fern from our API Definition.

from ....core.pydantic_utilities import UniversalBaseModel
import typing
from .face import Face
from .unconfigurable import Unconfigurable
from .prosody import Prosody
from .language import Language
from .ner import Ner
from ....core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class Models(UniversalBaseModel):
    """
    The models used for inference.
    """

    face: typing.Optional[Face] = None
    burst: typing.Optional[Unconfigurable] = None
    prosody: typing.Optional[Prosody] = None
    language: typing.Optional[Language] = None
    ner: typing.Optional[Ner] = None
    facemesh: typing.Optional[Unconfigurable] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
